import shutil

import terragit.terragrunt as terraconf
import gitlab
import os , fileinput
import base64
from terragit.gitlabFunctions import *
import re
from string import Template
import requests


class Create:
    def __init__(self):
        self.bcolor = terraconf.bcolors
        self.gitlab_functions = GitlabFunctions('glpat-BS_yy1VSPx2PKkbjokN3', 'https://gitlab.allence.cloud')

    def create_group(self, group_name):

        fondation_id ='1539'
        git_url = 'https://gitlab.allence.cloud'
        gitlab_token = os.environ.get('gitlab_token')
        gl = gitlab.Gitlab(url=git_url, private_token=gitlab_token)
        foundation = gl.projects.get(fondation_id)
        f1 = foundation.files.get(file_path='live/prod/clients-acp/chosa/git/main.tf', ref='master')
        f3 = foundation.files.get(file_path='live/prod/clients-acp/chosa/git/terragrunt.hcl', ref='master')
        file_content1 = base64.b64decode(f1.content).decode("utf-8")
        file_content3 = base64.b64decode(f3.content).decode("utf-8")
        path = "tf_files"
        isExist = os.path.exists(path)
        if not isExist:
            os.mkdir('terragit/tf_files')
        f1 = open("terragit/tf_files/main.tf", "w")
        f1.write(file_content1)
        f1.close()
        f2 = open("terragit/tf_files/terragrunt.hcl", "w")
        f2.write(file_content3)
        f2.close()
        f3 = open("terragit/tf_files/master.tfvars", "w")
        f3.write('')
        f3.close()

        file = open("terragit/tf_files/main.tf", "r")
        file1 = open("terragit/tf_files/main.tf", "r")
        replacement = ""
        replacement2 = ""
        old_module = 'source = "git::https://git@gitlab.com/commons-acp/terraform/gitlab/acp-instance.git?ref=v1.0.11"'
        new_module = 'source = "git@gitlab.allence.cloud:commons-acp/terraform/gitlab/acp-instance.git?ref=1.8.0"'

        for line in file:
            line = line.strip()
            changes = line.replace(old_module, new_module)
            replacement = replacement + changes + "\n"
        fout = open("terragit/tf_files/main.tf", "w")
        fout.write(replacement)
        fout.close()
        for line in file1:
            line = line.strip()
            changes2 = line.replace('project_name ="chosa"', 'project_name' + ' ="'+ group_name +'"')
            replacement2 = replacement2 + changes2 + "\n"
        file1.close()
        fout1 = open("terragit/tf_files/main.tf", "w")
        fout1.write(replacement2)
        fout1.close()

        old = "provider "+'"gitlab" '+"{"
        new = 'base_url = "https://gitlab.allence.cloud/api/v4/"'
        for line in fileinput.FileInput('terragit/tf_files/main.tf', inplace=True):
            if old in line :
                line += new + os.linesep
            print(line, end="")

        old = "root_id = dependency.parent.outputs.root_id"
        new = 'gitlab_token = get_env("gitlab_token")'

        for line in fileinput.FileInput('terragit/tf_files/terragrunt.hcl', inplace=True):
            if old in line :
                line += new + os.linesep
            print(line, end="")

        self.gitlab_functions.create_branch(fondation_id, group_name + "-patch" + str(fondation_id))

        path ='live/prod/clients-acp/'
        with open('terragit/tf_files/terragrunt.hcl', 'r') as file:
            data1 = file.read()

        with open('terragit/tf_files/main.tf', 'r') as file:
            data2 = file.read()

        with open('terragit/tf_files/master.tfvars', 'r') as file:
            data3 = file.read()

        data= {
            "branch": group_name + "-patch" + str(fondation_id),
            "commit_message": "add acp-instance to clients acp",
            "start_branch": group_name + "-patch" + str(fondation_id),
            "actions": [
                {
                    "action": "create",
                    "file_path": path+group_name+'/git/main.tf',
                    "content": data2
                },
                {
                    "action": "create",
                    "file_path": path+group_name+'/git/master.tfvars',
                    "content": data3
                },
                {
                    "action": "create",
                    "file_path": path+group_name+'/git/terragrunt.hcl',
                    "content": data1
                }
            ]
        }

        url = git_url + "/api/v4/projects/" + str(fondation_id) + "/repository/commits"
        add = requests.post(url, json=data, headers={'PRIVATE-TOKEN': gitlab_token})
        mr_request_creation = self.gitlab_functions.merge_request(fondation_id, group_name + "-patch" + str(fondation_id), "master")
        while mr_request_creation.json()['pipeline'] is None:
            mr = self.gitlab_functions.get_all_merge_request_by_project_id(fondation_id, mr_request_creation.json()['iid'])
            if mr['pipeline'] is not None:
                accept_mr = self.gitlab_functions.accept_merge_request(fondation_id, mr['iid'])
                if mr['state'] == 'merged':
                    shutil.rmtree(os.getcwd()+'/terragit/tf_files')
                    break





import requests



class GitlabFunctions:
    def __init__(self, gitlab_token,git_url):
        self.headers = {'PRIVATE-TOKEN': gitlab_token, 'Content-Type': 'application/json'}
        self.git_url = git_url

    def create_user(self, branch, path,  AwsUsername, keybase, project_id, action):

        postData = {
            'branch': branch,
            'commit_message': path,
            'actions': [
                {
                    'action': action,
                    'file_path': path + '/main.tf',
                    'content':
                        'provider "aws" {\n' +
                        '  region     = var.aws_region\n' +
                        '}\n' +
                        'module "user" {\n' +
                        '  source = "git::https://git@gitlab.com/commons-acp/terraform/aws/user.git?ref=v1.0.0"\n' +
                        '  name                              = "' + AwsUsername + '"\n' +
                        '  groups                            = [var.grp_devops_name]\n' +
                        '  force_destroy                     = true\n' +
                        '  pgp_key                           = "keybase:' + keybase + '"\n' +
                        '}'
                },
                {
                    'action': action,
                    'file_path': path + '/Makefile',
                    'content':
                        'credentials:\n' +
                        '\t@echo "####User name #####"\n' +
                        '\t@terraform output "user_name"\n' +
                        '\t@echo "#####PASSWORD####"\n' +
                        '\t@terraform output "encrypted_password" | base64 --decode | keybase pgp decrypt\n' +
                        '\t@echo ""\n' +
                        '\t@echo "#####TOKEN KEY####"\n' +
                        '\t@terraform output "access_key_id"\n' +
                        '\t@echo "####TOKEN SECRET#####"\n' +
                        '\t@terraform output "encrypted_secret_access_key" | base64 --decode | keybase pgp decrypt\n' +
                        '\t@echo ""\n' +
                        '\t@echo "Gardes ces acces secret utilise uniquement terraform pour les operations"\n'
                },
                {
                    'action': action,
                    'file_path': path + '/outputs.tf',
                    'content':
                        'output "user_name" {\n' +
                        '  value = module.user.aws_iam_user-credentials.name\n' +
                        '}\n' +
                        'output "access_key_id" {\n' +
                        '  value = module.user.aws_iam_user-credentials.access-key-id\n' +
                        '}\n' +
                        '#terraform output encrypted_password | base64 --decode | keybase pgp decrypt\n' +
                        'output "encrypted_secret_access_key" {\n' +
                        '  value = module.user.aws_iam_user-credentials.encrypted-secret-access-key\n' +
                        '}\n' +
                        '# terraform output encrypted_password | base64 --decode | keybase pgp decrypt\n' +
                        'output "encrypted_password" {\n' +
                        '  value = module.user.aws_iam_user-credentials.encrypted_password\n' +
                        '}'
                },
                {
                    'action': action,
                    'file_path': path + '/terragrunt.hcl',
                    'content':
                        'include {\n' +
                        '  path = find_in_parent_folders()\n' +
                        '}\n' +
                        '\n' +
                        '\n' +
                        '\n' +
                        'dependency "groups"{\n' +
                        '  config_path = "../../groups"\n' +
                        '}\n' +
                        'inputs = {\n' +
                        '  grp_devops_name = dependency.groups.outputs.grp_devops_name\n' +
                        '  aws_region = "eu-west-1"\n' +
                        '}'
                },
                {
                    'action': action,
                    'file_path': path + '/variables.tf',
                    'content':
                        'variable "aws_region" {\n' +
                        '  description = "AWS region"\n' +
                        '}\n' +
                        'variable "grp_devops_name" {\n' +
                        '  description = "Group devops name"\n' +
                        '}\n'
                }

            ]
        }

        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/repository/commits"
        add = requests.post(url, json=postData, headers=self.headers)


    def create_branch(self,  project_id, branch_name):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/repository/branches?branch=' + branch_name+'&ref=master'
        return requests.post(url, "", headers=self.headers)

    def get_all_project_branches(self, project_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/repository/branches?per_page=100"
        return requests.get(url, headers=self.headers).json()

    def delete_branch(self, project_id, branchName):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/repository/branches/"+str(branchName)
        return requests.delete(url, headers=self.headers)

    def get_single_commit(self,  project_id, commit_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/repository/commits/' + str(commit_id)
        return requests.get(url,  headers=self.headers).json()

    def get_last_mr_commit_id(self,  project_id, mr_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/' + str(mr_id) + '/commits'
        return requests.get(url, headers=self.headers).json()

    def get_full_commit_by_id(self,  project_id, commit_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/repository/commits/' + str(commit_id)
        return requests.get(url,  headers=self.headers)

    def get_pipeline_jobs(self,  project_id, last_pipeline_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/pipelines/' + str(last_pipeline_id)+'/jobs'
        return requests.get(url,  headers=self.headers).json()

    def merge_request(self,  project_id, branch_name, main_branch):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/?id='+str(project_id)+'&title=auto_mr_'+branch_name+'&source_branch='+branch_name+'&target_branch='+main_branch
        return requests.post(url, "", headers=self.headers)

    def accept_merge_request(self, project_id, mr_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/' + str(mr_id)+'/merge?should_remove_source_branch=true'
        return requests.put(url, "", headers=self.headers)

    def delete_merge_request(self, project_id, mr_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/merge_requests/" + str(mr_id)
        return requests.delete(url, headers=self.headers)

    def get_all_project_merge_requests(self, project_id,  created_before):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/?state=opened&created_before='+str(created_before)+'&per_page=1'
        return requests.get(url,  headers=self.headers).json(),

    def get_all_merge_request_by_project_id(self, project_id, mr_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/' + str(mr_id)
        return requests.get(url,  headers=self.headers).json()

    def get_all_merge_request(self, project_id, created_before):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/?state=all&created_before='+str(created_before)+'&per_page=10'
        return requests.get(url,  headers=self.headers).json()

    def get_all_merge_request_by_page(self, project_id, created_before, limit):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/?state=opened&created_before='+str(created_before)+'&per_page=20&page=' + str(limit)
        return requests.get(url,  headers=self.headers).json()

    def get_number_merge_request_project(self, project_id, created_before, limit, page):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/?state=opened&created_before='+str(created_before)+'&per_page='+str(limit) +'&page=' + str(page)
        return requests.get(url,  headers=self.headers).json()

    def get_number_merge_request_group(self, group_id, created_before, limit, page):
        url = self.git_url + "/api/v4/groups/" + str(group_id) + '/merge_requests/?state=opened&created_before='+str(created_before)+'&per_page='+str(limit) +'&page=' + str(page)
        return requests.get(url,  headers=self.headers).json()

    def update_mr_state(self , project_id, iid):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/merge_requests/' + str(iid)
        return requests.put(url, data='{"state_event": "close"}', headers=self.headers).json()

    def get_all_group_merge_requests(self, group_id,  created_before):
        url = self.git_url + "/api/v4/groups/" + str(group_id) + '/merge_requests/?state=opened&created_before='+str(created_before)
        return requests.get(url,  headers=self.headers).json()

    def get_all_group_merge_requests_per_page(self, group_id,  created_before, limit):
        url = self.git_url + "/api/v4/groups/" + str(group_id) + '/merge_requests/?state=opened&created_before='+str(created_before)+'&per_page=20&page=' + str(limit)
        return requests.get(url,  headers=self.headers).json()

    def get_job_trace(self, project_id, job_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + '/jobs/' + str(job_id)+'/trace'
        return requests.get(url, headers=self.headers).text

    def get_services(self, project_id,  page):
        url = self.git_url + "/api/v4/projects/" + str(
            project_id) + "/repository/tree?recursive=true&per_page=20&page=" + str(page)
        return requests.get(url, headers=self.headers).json()

    def get_file_content(self, project_id, path_replaced):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/repository/files/" + path_replaced + '?ref=master'
        return requests.get(url, headers=self.headers).json()

    def get_project_by_id(self, project_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id)
        return requests.get(url, headers=self.headers).json()

    def get_group_by_id(self, group_id):
        url = self.git_url + "/api/v4/groups/" + str(group_id)
        return requests.get(url, headers=self.headers).json()

    def get_projects_of_group(self, group_id, page, per_page):
        url = self.git_url + "/api/v4/groups/" + str(group_id) + "/projects?per_page=" + str(per_page) + "&page=" + str(page)
        return requests.get(url, headers=self.headers).json()

    def current_group_project(self, project_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id)
        web_url_project = requests.get(url, headers=self.headers).json()['web_url']
        name_project = requests.get(url, headers=self.headers).json()['name']
        url_group = self.git_url + "/api/v4/projects/" + str(project_id) + "/groups"
        url_groups = requests.get(url_group, headers=self.headers).json()
        for group in url_groups:
            if str(group['web_url']).replace("/groups", "")+"/"+name_project == web_url_project:
                return group

    def check_project_infra_exist(self, group_id):
        url = self.git_url + "/api/v4/groups/" + str(group_id) + "/projects"
        projects = requests.get(url, headers=self.headers).json()
        for pr in projects:
            if pr['name'] == "infra":
                return pr['id']

    def get_project_pipeline(self, project_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/pipelines"
        return requests.get(url, headers=self.headers).json()

    def check_with_user_folder(self, project_id):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/repository/tree?path=live/aws/global/iam/users"
        return requests.get(url, headers=self.headers).json()

    def get_user_tfstate(self, project_id, gitlab_username):
        url = self.git_url + "/api/v4/projects/" + str(project_id) + "/terraform/state/aws_global_iam_users_" + str(gitlab_username)
        return requests.get(url, headers=self.headers).json()

    # def delete_file_of_repo(self):
    #     print()





import json
import os
import re
import gitlab
import pandas as pd
from pandas import option_context

import terragit.terragrunt as terraconf


class Status:
    def __init__(self):
        self.bcolor = terraconf.bcolors

    def module(self):

        git_url = 'https://gitlab.allence.cloud'
        gitlab_token = os.environ.get('gitlab_token')

        mylist = []
        path_absolute = os.getcwd()
        for root, dirs, files in os.walk(path_absolute + "/live"):
            mylist.append(root)

        directories = []
        for r in mylist:
            if ".terraform" not in r and "build-front" not in r:
                file_exists = r + '/main.tf'
                isexist = os.path.exists(file_exists)
                if isexist:
                    directories.append(file_exists)
        modules = []
        tags = []

        for d in directories:
            with open(d) as temp_f:
                datafile = temp_f.readlines()
            for line in datafile:
                if 'module' in line:
                    for ligne in datafile:
                        if ligne.find('source') != -1 and ligne.find('git') != -1:
                            var = ligne.split("@", 1)[1]
                            new_address = re.sub(':', '/', var)
                            new_module = re.sub('"', "", new_address)
                            tags.append(new_module)

                            final = new_module.split('?', 1)[0]

                            if '.git' in final:
                                first_final = re.sub('.git', '', final)
                                last_final = first_final.split('/', 1)[1]
                                modules.append(last_final)
                            else:
                                last_final = final.split('/', 1)[1]
                                modules.append(last_final)
        last_tag = []
        gl = gitlab.Gitlab(url=git_url, private_token=gitlab_token)
        for m in modules:
            project_module = gl.projects.get(m)
            tag = project_module.tags.list()
            if len(tag) == 0:
                last_tag.append('no tag found')
            else:
                x = tag[0].attributes['name']
                last_tag.append(x)
        json_array = []
        for i in range(len(last_tag)):
            data = {}
            data['path'] = tags[i]
            data['recent_tag'] = last_tag[i]
            json_data = json.dumps(data)
            json_array.append(json_data)

        dictionary = []
        for j in json_array:
            dictionary.append(json.loads(j))

        df = pd.DataFrame(dictionary)
        with option_context('display.max_colwidth', 400):
            # display the dataframe
            print(df)




import sys
from os import listdir
from os.path import isdir, join
import gitlab
import os
import pathlib
import subprocess
import shutil


import terragit.terragrunt as terraconf
from terragit.gitlabFunctions import *


pathList=[]
failedloglist=[]
logsFolder=""
printLogs=False



class Validate:
    def __init__(self):
        self.bcolor = terraconf.bcolors

    def validate_ci(self):
        git_url="https://gitlab.allence.cloud"
        if len(sys.argv) > 1:
            git_url = sys.argv[1]
        logsFolder=pathlib.Path("logs").absolute().as_posix()
        failedlogsFolder=pathlib.Path("failedlogs").absolute().as_posix()
        idMr = os.environ.get('CI_MERGE_REQUEST_IID')
        idProject=os.environ.get('CI_PROJECT_ID')
        gitlab_token=os.environ.get('gitlab_token')
        ci_commit_title=os.environ.get('CI_COMMIT_TITLE')
        gl = gitlab.Gitlab(url="https://gitlab.allence.cloud",private_token = gitlab_token)

        project = gl.projects.get(idProject)
        mr= project.mergerequests.get(idMr)
        folderList=[]
        mrchange=mr.changes()
        changes = mrchange['changes']
        if (len(changes)==0):
            if not isdir(pathlib.Path(ci_commit_title).absolute().as_posix()):
                print(self.bcolor.FAIL+ci_commit_title+" is not valid path" + self.bcolor.ENDC)
                failedloglist.append(ci_commit_title)
            else:
                ci_mr_titlePath=pathlib.Path(ci_commit_title).absolute().as_posix()
                printLogs=True
                self.validate_all(ci_mr_titlePath,logsFolder,printLogs)
        else:
            for change in changes:
                newPath=change['new_path']
                if not ("live/") in newPath:
                    print(pathlib.Path(newPath).absolute().as_posix()+self.bcolor.WARNING +" OUT of SCOPE"+ self.bcolor.ENDC)
                else:
                    pathh=pathlib.Path(newPath).parent.absolute().as_posix()
                    folderList.append(pathh)
        mylist = list(dict.fromkeys(folderList))
        TG_OUTPUT_LIMIT =3
        if os.environ.get("TG_OUTPUT_LIMIT")!=3 and os.environ.get("TG_OUTPUT_LIMIT")!=None :
            TG_OUTPUT_LIMIT = os.environ.get("TG_OUTPUT_LIMIT")

        if len(mylist)<=int(TG_OUTPUT_LIMIT) and any(".hcl" in l for l in mylist):
            printLogs=True
        for path in mylist:
            if isdir(path):
                self.validate_all(path,logsFolder,printLogs)
        if failedloglist:
            for message in failedloglist:
                logfileName=message.split("live/")[1].replace("/","_")
                os.chdir(failedlogsFolder)
                shutil.move(logsFolder+"/"+logfileName+".log", "failed_"+logfileName+".log")
            sys.exit(1)


    def validate_all(self,path,logsFolder,printLogs):
        if  path in pathList:
            return
        else:
            pathList.append(path)
            only_tf_files=[f for f in  sorted(listdir(path)) if not isdir(join(path, f)) and f.endswith('.tf')]
            if len(only_tf_files)==0:
                onlyDirectories = [d for d in sorted(listdir(path)) if isdir(join(path, d)) and d !=".terraform" and d != ".terragrunt-cache"]
                if(len(onlyDirectories) > 0):
                    for i in range(0, len(onlyDirectories)):
                        self.validate_all(path+"/"+onlyDirectories[i],logsFolder,printLogs)
                return
            os.chdir(path)
            logfileName=path.split("live/")[1].replace("/","_")

            state_list = ("terragrunt validate -no-color 2>&1 | tee " +logsFolder+"/"+logfileName+".log")
            popen = subprocess.Popen(state_list, stdout = subprocess.PIPE, shell = True, encoding = 'utf8',env=os.environ.copy())
            lines = popen.stdout.readlines()
            popen.stdout.close()
            print(self.bcolor.OKBLUE+path+":" +self.bcolor.ENDC)
            for line in lines:
                if (printLogs):
                    print(line)
                else:
                    if ("Error" in line):
                        lint=lines[(lines.index(line)):(len(lines))]
                        for l in lint:
                            print(l.replace('\n',''))
                        failedloglist.append(path)
                        print(self.bcolor.FAIL +" COULDNT PROCESS"+ self.bcolor.ENDC)
                        return
                if("Success! The configuration is valid" in line):
                    print(self.bcolor.OKGREEN +"Success."+ self.bcolor.ENDC)
                    return
            print(self.bcolor.FAIL +"configuration is not valid."+ self.bcolor.ENDC)
            failedloglist.append(path)





import signal
import subprocess
import sys
from os import listdir
from os.path import isdir, join
import gitlab
import os
import pathlib
import terragit.terraConf as terra_conf

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    WHITE   = '\033[37m'


class Unbuffered(object):
    def __init__(self, stream):
        self.stream = stream

    def write(self, data):
        self.stream.write(data)
        self.stream.flush()

    def writelines(self, datas):
        self.stream.writelines(datas)
        self.stream.flush()

    def __getattr__(self, attr):
        return getattr(self.stream, attr)


class terragrunt:
    def __init__(self, verbose):

        self.pathList = []
        self.failedloglist = []
        self.logfileName = []
        self.state_list = ""
        self.logsFolder = ""
        self.failedlogsFolder = ""
        self.verbose = verbose
        if self.verbose:
            if not os.path.isdir(os.getcwd() + "/logs"):
                os.makedirs(os.getcwd() + "/logs")
            if not os.path.isdir(os.getcwd() + "/failedlogs"):
                os.makedirs(os.getcwd() + "/failedlogs")
            self.logsFolder = pathlib.Path("logs").absolute().as_posix()
            self.failedlogsFolder = pathlib.Path("failedlogs").absolute().as_posix()

    def getAllFolder(self, path):
        if path in self.pathList:
            return
        else:

            only_tf_files = [f for f in sorted(listdir(path)) if not isdir(join(path, f)) and f.endswith('.tf')]
            if len(only_tf_files) == 0:
                onlyDirectories = [d for d in sorted(listdir(path)) if
                                   isdir(join(path, d)) and d != ".terraform" and d != ".terragrunt-cache"]
                if (len(onlyDirectories) > 0):
                    for i in range(0, len(onlyDirectories)):
                        self.getAllFolder(path + "/" + onlyDirectories[i])
            else:
                self.pathList.append(path)
        return self.pathList

    def executeTerragrunt(self, command, path, logsFolder, verbose):

        terraconf = terra_conf.TerraConf()
        content = terraconf.get_file_content()
        current_project = terraconf.get_selected_project(content)
        logfileName = path.split("live/")[1].replace("/", "_")
        # profile = "dev02"
        # if("20-int" in path) : profile = "int"
        # if("30-uat" in path) : profile = "uat"
        # if("31-uatm" in path) : profile = "uat"
        # if("32-miscuat" in path) : profile = "miscuat"
        # if("42-miscprod" in path) : profile = "miscprod"
        # if("40-prod" in path) : profile = "prod"
        # if("41-proddr" in path) : profile = "prod"
        if verbose:
            self.state_list = (
                        " export AWS_ACCESS_KEY_ID=" + current_project['aws_credentials']['access_key'] + " && export AWS_SECRET_ACCESS_KEY=" + current_project['aws_credentials']['secret_access_key'] + " && export gitlab_token=" + content['gitlab_token']  + " && export gitlab_user=" + content['gitlab_user']  + " &&  terragrunt " + command + " -no-color 2>&1 | tee " + logsFolder + "/" + logfileName + ".log")
        else:

            self.state_list = (" export AWS_ACCESS_KEY_ID=" + current_project['aws_credentials']['access_key'] + " && export AWS_SECRET_ACCESS_KEY=" + current_project['aws_credentials']['secret_access_key'] + " && export gitlab_token=" + content['gitlab_token']  + " && export gitlab_user=" + content['gitlab_user']   + " && terragrunt " + command + " -no-color  2>&1 ")
        return self.state_list


    def printlog(self, command, pathList, logsFolder, verbose):
        sys.stdout = Unbuffered(sys.stdout)
        print("pathList", pathList)
        for path in pathList:
            print(bcolors.OKBLUE + path + ":" + bcolors.ENDC)
            os.chdir(path)
            self.executeTerragrunt(command, path, logsFolder, verbose)
            popen = subprocess.Popen(self.state_list, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                     encoding='utf8')
            lines = popen.communicate(input='\n')[0].split("\n")
            for line in lines:
                print("line", line);
                if ("No changes. Infrastructure is up-to-date." in line):
                    print(bcolors.OKGREEN + "  : No changes. Infrastructure is up-to-date." + bcolors.ENDC)
                    break
                if ("No changes. Your infrastructure matches the configuration." in line):
                    print(bcolors.OKGREEN + "  : No changes. Your infrastructure matches the configuration." + bcolors.ENDC)
                    break
                if ("Plan:" in line):
                    print(bcolors.WARNING + line + bcolors.ENDC)
                    break
                if ("will be updated in-place" in line):
                    print(line.replace('\n', ''))
                if ("will be created" in line):
                    print(line.replace('\n', ''))
                if ("must be replaced" in line):
                    print(line.replace('\n', ''))
                    lint = lines[(lines.index(line)):(len(lines))]
                    lin = lint[0:(lint.index(''))]
                    for l in lin:
                        if ("forces replacement" in l):
                            print(l.replace('\n', ''))
                            print(lin[(lin.index(l)) + 1])
                if ("will be destroyed" in line):
                    print(line.replace('\n', ''))
                errors = ["Hit multiple errors", "errors", "Error", "Unable to determine underlying exit code"]

                if any(line.startswith(error) for error in errors):
                    lint = lines[(lines.index(line)):(len(lines))]
                    for l in lint:
                        print(l.replace('\n', ''))
                    print(bcolors.FAIL + " COULDNT PROCESS" + bcolors.ENDC)
                    self.failedloglist.append(path)
                    break
                if ("Success! The configuration is valid" in line):
                    print(bcolors.OKGREEN + "Success." + bcolors.ENDC)
                    break
                if ("Apply complete! Resources:" in line):
                    print(bcolors.OKGREEN + line + bcolors.ENDC)
                    break
                # output  messsage
                if command == "output":
                    if ("will detect it and remind you to do so if necessary" in line):
                        lin = lines[(lines.index(line) + 1):(len(lines))]
                        for l in lin:
                            if ("The state file either has no outputs defined" in l):
                                print(
                                    bcolors.WARNING + "  : no outputs defined or all the defined outputs are empty." + bcolors.ENDC)
                                break
                            else:
                                print(l)
                    break

            else:
                print(bcolors.FAIL + " COULDNT PROCESS" + bcolors.ENDC)
                self.failedloglist.append(path)



import argparse
import os
import sys

import terragit.addUserGitlabRepo as add_user_gitlab_repository
import terragit.clone as clone
import terragit.comapare as compare
import terragit.gitlabFunctions as GitlabFunctions
import terragit.projectStatus as ProjectStatus
import terragit.terraClean as terraClean
import terragit.terraConf as terra_conf
import terragit.terracommand as terracommande
import terragit.terradocs as terradocse
import terragit.terragrunt as terragrunt
import terragit.utilsFunctions as utils
import terragit.terraValidate as terraValidate
import terragit.mrchanges as mrchanges
import terragit.apply as applychanges
import terragit.terraStatus as terraStatus
import terragit.createGroup as creation
import terragit.ValidateCi as validation


def main():

    parser = argparse.ArgumentParser()
    colors = terragrunt.bcolors()
    terraconf = terra_conf.TerraConf()
    parser.add_argument("-u", "--gitlab_url", dest="gitlab_url", default="https://gitlab.allence.cloud", help="gitlab url")
    verif = terraconf.verif_file_and_credentials_existence()
    token_ci_id = os.getenv("CI_JOB_TOKEN")

    if token_ci_id is None:
        if (sys.argv[1] in ["config"]) or (verif != "false"):
            if sys.argv[1] == "config":
                terraconf.init_file()


            if sys.argv[1] == "docs":
                parser.add_argument("-m", "--module", action="store_true", help="module")
                parser.add_argument("-l", "--live", action="store_true", help="live")
                parser.add_argument("-o", "--output", dest="output_path", default="./", help="output path")
                parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="id of project")
                parser.add_argument(sys.argv[1])

                args = parser.parse_args()
                terradoc = terradocse.terradoc(args.gitlab_url, verif["gitlab_token"], args.project_id)
                terradoc.docs(args.module, args.live)

            if sys.argv[1] == "aws-adduser":
                group_id = os.getenv("GROUP_ID")
                parser.add_argument("-gu", "--gitlab_username", dest="gitlab_username", default=None,
                                    help="member gitlab username")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                add_user_aws = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
                add_user_aws.add_aws_account(group_id, args.gitlab_username)

            if sys.argv[1] == "plan":

                parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                                    help="your group name")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                order = terracommande.terracommand("","","","","","","","")
                order.terragrunt_plan(args.group_name)

            if sys.argv[1] == "create-group":
                parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                            help="your group name")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                group_creation = creation.Create()
                group_creation.create_group(args.group_name)

            if sys.argv[1] == "clone":

                parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="group id")
                parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="id of project")
                parser.add_argument("-ip", "--infra_project_id", dest="infra_project_id",
                                    default=None, help="id of the infrastructure project")
                parser.add_argument("-path", "--path", dest="path", default="./",
                                    help="path in which the project will be cloned")

                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                print(colors.OKCYAN, "Cloning ...")
                clonn = clone.Clone(verif["gitlab_token"], args.gitlab_url, args.path)
                clonn.clone_projects(args.project_id, args.group_id)
                print(colors.OKCYAN, "Checking the existance of IAM user ...")
                utilsFunctions = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
                gt = GitlabFunctions.GitlabFunctions(verif["gitlab_token"], args.gitlab_url)
                service_path = utilsFunctions.check_content_existance_in_serviceGr(args.group_id,args.infra_project_id,
                                                                                   'https://git@gitlab.com/commons-acp/terraform/aws/user.git','name="' + verif["gitlab_user"] + '"')

                if "content_found" in service_path:
                    print(colors.OKGREEN, "IAM user found!")

                    terraconf.add_credentials_to_group(args.gitlab_url, verif["gitlab_token"], args.group_id,
                                                       args.infra_project_id, args.path, "/"+service_path[29:service_path.rindex("/")], "true")
                else:
                    print("iam user not found")
                    terraconf.add_credentials_to_group(args.gitlab_url, verif["gitlab_token"],args.group_id,
                                                       args.infra_project_id, args.path, "", "false")

            ####
            if sys.argv[1] == "compare":
                print("comparing infra")
                parser.add_argument("-I", "--infra", dest="infra", default=None, help="path to infra")
                parser.add_argument("-N", "--infra2", dest="infra2", default=None, help="path to infra")
                parser.add_argument(sys.argv[1])


                args = parser.parse_args()
                list = compare.Compare(verif["gitlab_token"], args.gitlab_url)
                #
                list.compare_tf_states(args.infra, args.infra2)

            if sys.argv[1] == "validate":
                print("argv", sys.argv)
                print("validate infra")
                parser.add_argument("-p", "--path_local", dest="local_path", default=None, help="local path to infra")
                parser.add_argument("-P", "--path", dest="path", default=None, help="path to infra")
                parser.add_argument("-prId", "--project_id", dest="project_id", help="project id")
                parser.add_argument(sys.argv[1])

                args = parser.parse_args()
                list = terraValidate.TerraValidate(verif["gitlab_token"], args.gitlab_url)
                if(sys.argv[2] == "-p"):
                    list.validate_projects(args.local_path)
                else:
                    list.validate_distant(args.project_id, args.path)
            if sys.argv[1] == "adduser":
                parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="group id")
                parser.add_argument("-p", "--project_id", dest="project_id", default=None,
                                    help="id of infrastructure project")
                parser.add_argument("-gu", "--gitlab_username", dest="gitlab_username", default=None,
                                    help="member gitlab username")
                parser.add_argument("-lvl", "--access_level", dest="access_level", default=None,
                                    help="User access level can be guest, reporter, developer, maintainer or owner")
                parser.add_argument("-k", "--keybase_username", dest="keybase_username", default=None,
                                    help="user keybase username")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                add_user_gitlab_repo = add_user_gitlab_repository.AddUserGitlabRepo(verif["gitlab_token"], args.gitlab_url)
                exist = add_user_gitlab_repo.existance_gitlab(args.project_id, args.group_id, args.gitlab_username)
                if exist[0] == 'user exist in gitlab project & ':
                    print(colors.FAIL + "user exist in gitlab project")
                else:
                    add_user_gitlab_repo.add_user_repository(args.project_id, args.group_id, args.gitlab_username, args.access_level,
                                                             exist, args.keybase_username)

            if sys.argv[1] == "list":
                content = terraconf.get_file_content()
                terraconf.list_projects(content)

            if sys.argv[1] == "currentProject":
                content = terraconf.get_file_content()
                terraconf.get_selected_project(content)

            if sys.argv[1] == "switch":
                parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="gitlab group id")
                parser.add_argument("-gn", "--group_name", dest="group_name", default=None, help="gitlab group name")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                content = terraconf.get_file_content()
                terraconf.switch_project(content, args.group_id, args.group_name)

            if sys.argv[1] == "clean":
                parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="gitlab group id")
                parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="gitlab project id")
                parser.add_argument("-t", "--time", dest="time", default=90, help="provide time to delete "
                                                                                  "branches/merge-requests older then time provided")
                parser.add_argument("-mr", "--mr", dest="mr", help="set it to true if you want to delete Mr else false")
                parser.add_argument("-b", "--branches", dest="branches", help="set it to true if you want to delete "
                                                                              "branches else false")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                terra_clean = terraClean.TerraClean(verif["gitlab_token"], args.gitlab_url)
                terra_clean.clean(args.group_id, args.project_id, int(args.time), args.mr, args.branches)

            if sys.argv[1] == "help":
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                util = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
                util.help()

            if sys.argv[1] == "status":
                parser.add_argument("-dir", "--directory", dest="directory", default=None, help="gitlab path")
                parser.add_argument(sys.argv[1])
                args = parser.parse_args()
                status = ProjectStatus.ProjectStatus(colors)
                status.gitstatus(args.directory)

            if sys.argv[1] == "project-list":
                list = clone.Clone('glpat-BS_yy1VSPx2PKkbjokN3', 'https://gitlab.allence.cloud/', '')
                list.list_project()

        else:
            print(colors.FAIL, "terragit is not initialised please run `terragit config` command first")
    else:
        if sys.argv[1] == "plan":

            parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                                help="your group name")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            plan = mrchanges.mr_changes()
            plan.plan_ci()
        if sys.argv[1] == "apply":
            parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                                help="your group name")
            parser.add_argument(sys.argv[1])
            apply = applychanges.apply_changes()
            apply.apply_ci()

        if sys.argv[1] == "validate":
            validate = validation.Validate()
            validate.validate_ci()

        if sys.argv[1] == "status":
            parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                                help="your group name")
            parser.add_argument(sys.argv[1])
            status = terraStatus.Status()
            status.module()




if __name__ == '__main__':
    main()



import os
import terragit.terraConf as terraconf
import terragit.clone as clone


class TerraValidate:
    def __init__(self, gitlab_token, git_url):

        self.gitlab_token = gitlab_token
        self.git_url = git_url

    def validate_projects(self, local_path):
        # content = terraconf.TerraConf.get_file_content()
        print("hello from validate")
        # terraconf.TerraConf.check_project_existance(content, group_id, group_name)
        cmd = "touch validate.txt && cd " + local_path + " && terragrunt validate "
        print("cmd", cmd)
        os.system(cmd)
    def validate_distant(self, projectId, path):
        clone.Clone.clone_projects(self, projectId, None)
        cmd= "pwd"
        pwd = os.popen(cmd).read().rstrip()
        local_path = pwd+"/"+path
        print("local", local_path)
        self.validate_projects(local_path)


import json
import os
import shutil
from os.path import exists

import gitlab as gitlab

# from tqdm import tqdm

from terragit.terragrunt import *
import terragit.terraConf as terra_conf



class terracommand(terragrunt, bcolors):

    def __init__(self, idProject, idCommit, idMr, gitlab_token, git_url, directory, verbose, ci_commit_title=""):
        super().__init__(verbose)
        self.idProject = idProject
        self.idCommit = idCommit
        self.idMr = idMr
        self.gitlab_token = gitlab_token
        self.git_url = git_url
        self.ci_commit_title = ci_commit_title
        self.directory = directory
        self.verbose = verbose
        self.terraconf = terra_conf.TerraConf()

    # def terragruntCommand(self, command):
    #     mylist = []
    #     if self.directory != None:
    #         mylist.append(self.directory)
    #     else:
    #         gl = gitlab.Gitlab(self.git_url, private_token=self.gitlab_token)
    #         project = gl.projects.get(self.idProject)
    #         if self.idCommit != None:
    #             commit = project.commits.get(self.idCommit)
    #             diff = commit.diff()
    #         if self.idMr != None:
    #             mr = project.mergerequests.get(self.idMr)
    #             diff = mr.changes()['changes']
    #         folderList = []
    #         if (len(diff) == 0):
    #             if not isdir(pathlib.Path(self.ci_commit_title).absolute().as_posix()):
    #                 print(bcolors.FAIL + self.ci_commit_title + " is not valid path" + bcolors.ENDC)
    #             else:
    #                 print("len(diff)==0 else ")
    #                 self.ci_commit_titlePath = pathlib.Path(self.ci_commit_title).absolute().as_posix()
    #                 self.pathList.append(self.ci_commit_titlePath)
    #                 self.printlog(command, self.pathList, self.logsFolder, self.verbose)
    #         else:
    #             for change in diff:
    #                 print("change ", change)
    #                 newPath = change['new_path']
    #                 if not ("live/") in newPath:
    #                     print(pathlib.Path(
    #                         newPath).absolute().as_posix() + bcolors.WARNING + " OUT of SCOPE" + bcolors.ENDC)
    #                 else:
    #                     pathh = pathlib.Path(newPath).parent.absolute().as_posix()
    #                     folderList.append(pathh)
    #
    #         mylist = list(dict.fromkeys(folderList))
    #         print("mylist ", mylist)
    #
    #     for path in mylist:
    #         print("mylist for ", mylist)
    #         if (isdir(path)):
    #             print("is dir ")
    #             self.getAllFolder(path)
    #             if command == "changes":
    #                 print(mylist)
    #                 return mylist
    #     self.printlog(command, self.pathList, self.logsFolder, self.verbose)
    #     if self.failedloglist:
    #         if self.verbose:
    #             for message in self.failedloglist:
    #                 logfileName = message.split("live/")[1].replace("/", "_")
    #                 os.chdir(self.failedlogsFolder)
    #
    #                 shutil.move(self.logsFolder + "/" + logfileName + ".log", "failed_" + logfileName + ".log")
    #         sys.exit(1)

    def terragrunt_plan(self, group_name):

        token_ci_id = os.getenv("CI_JOB_TOKEN")
        path_absolute = ""
        # if token_ci_id is None:
        #     path_absolute = "/home/hiba/Desktop/TERRAGIT/infra"
        #     # path = self.terraconf.get_file_content()
        #     # for root, dirs, files in os.walk(path[group_name]['path']):
        #     #     list.append(root)
        #     # i = 0
        #     # while str(group_name)+"-grp/infra" not in list[i]:
        #     #     i += 1
        #     #     if str(group_name)+"-grp/infra" in list[i]:
        #     #         path_absolute = list[i]
        #     #         break
        if token_ci_id is None:
            path_absolute ="/home/hiba/Desktop/TERRAGIT/infra"

        subdir = []
        for root, subdirs, files in os.walk(path_absolute):
            if os.path.lexists(os.path.join(root, "terragrunt.hcl")):
                subdir.append(root)

        cwd = [
            '/home/hiba/Desktop/TERRAGIT/infra/live/aws/global/iam/users/hiba',
               '/home/hiba/Desktop/TERRAGIT/infra/live/aws/1-dev/app-dev.chosa.allence.cloud/api-acm-validation',
               '/home/hiba/Desktop/TERRAGIT/infra/live/aws/global/iam/users/kacem.yedes'
               ]

        if not os.path.exists("logs"):
            os.mkdir("logs")
        if not os.path.exists("failedlogs"):
            os.mkdir("failedlogs")

        # number_of_task = len(cwd)
        # progress_bar = tqdm(total=number_of_task)
        process = 'source ~/my_profile && terragrunt plan -out=tfplan'
        process1 = 'source ~/my_profile ; terragrunt show -json tfplan > tfplan.json'
        processes = []
        processes1 = []

        for c in cwd:
            if "webapp" not in c:
                name_file = c.split("live/")[1].replace("/","_")+".log"
                logs = open(os.path.expanduser(os.path.join("logs/"+name_file)), mode='w+')
                failedlogs = open(os.path.expanduser(os.path.join("failedlogs/"+name_file)), mode='w+')
                proc = subprocess.Popen(["/bin/bash", "-c", process], bufsize=8192, stdin=None, stdout=logs, stderr=failedlogs, cwd=c)
                processes.append(proc)
        for p in processes:
            # progress_bar.update()
            p.wait()

        for c in cwd:
            if "webapp" not in c:
                proc1 = subprocess.Popen(["/bin/bash", "-c", process1], bufsize=8192, stdin=None, stdout=subprocess.PIPE ,stderr=subprocess.PIPE, cwd=c)
                processes1.append(proc1)
        for p in processes1:
            p.wait()

        files_sucess = []
        files_success = []
        files_error = []

        for i in range(len(cwd)):
            if exists(os.path.expanduser(os.path.join(cwd[i], "tfplan.json"))):
                file = open(os.path.expanduser(os.path.join(cwd[i], "tfplan.json")))
                data = file.readlines()
                if len(data) > 2:
                    json_object = json.loads(data[2])
                    file.close()
                    files_sucess.append(json_object['resource_changes'])
                    files_success.append({cwd[i]: json_object['resource_changes']})

                else:
                    files_error.append(cwd[i])


        added = 0
        changed = 0
        deleted = 0
        no_op = 0

        for i in range(len(files_sucess)):
            mykeys = list(files_success[i].keys())
            print(bcolors.ENDC, "You re working in folder", bcolors.OKBLUE, mykeys[0])
            for j in files_sucess[i]:
                if j['change']['actions'] == ['create']:
                    added += 1
                    print(bcolors.OKGREEN, "+", bcolors.ENDC, ' ressource', j['address'], 'will be added')
                if j['change']['actions'] == ['update']:
                    changed += 1
                    print(bcolors.WARNING,"~", bcolors.ENDC, ' ressource', j['address'], 'will be updated in-place')
                if j['change']['actions'] == ['delete']:
                    deleted += 1
                    print(bcolors.FAIL ,"-", bcolors.ENDC, ' ressource', j['address'], 'will be destroyed')
                if j['change']['actions'] == ['no-op']:
                    no_op += 1
                    if len(files_sucess) == no_op:
                        print(bcolors.OKGREEN +"   No changes. Infrastructure is up-to-date in this folder")
        print("Plan:", bcolors.OKGREEN, added, bcolors.ENDC, "to add, ", bcolors.WARNING,changed, bcolors.ENDC, " to change, ", bcolors.FAIL, deleted, bcolors.ENDC, " to destroy.")

        for f in files_error:
            print(bcolors.ENDC, "You re working in folder", bcolors.OKBLUE, f)
            print(bcolors.FAIL, 'error , you can consult your failedlogs file')

    def terragit_plan(self):

        token_ci_id = os.getenv("CI_JOB_TOKEN")
        path = os.getenv("my_path")
        path_absolute = path
        subdir = []
        for root, subdirs, files in os.walk(path_absolute):
            if os.path.lexists(os.path.join(root, "terragrunt.hcl")):
                subdir.append(root)
        cwd = [
            # '/builds/clients-acp/202201/chosa-grp/infra/live/aws/global/iam/users/hiba',
            '/builds/clients-acp/202201/chosa-grp/infra/live/aws/global/iam/users/hiba.jaouadi',
            # '/builds/clients-acp/202201/chosa-grp/infra/live/aws/1-dev/app-dev.chosa.allence.cloud/api-acm-validation'
        ]
        number_of_task = len(cwd)
        # progress_bar = tqdm(total=number_of_task)
        process1 = ' terragrunt plan -out=tfplan'
        process2 = ' terragrunt show -json tfplan > tfplan.json'
        processes = []
        logs = pathlib.Path("logs").absolute().as_posix()
        failedlogs = pathlib.Path("failedlogs").absolute().as_posix()
        for c in cwd:
            name_file = c.split("live/")[1].replace("/","_")+".log"
            logs = open(os.path.expanduser(os.path.join(logs+"/"+name_file)), mode='w+')
            failedlogs = open(os.path.expanduser(os.path.join(failedlogs+"/"+name_file)), mode='w+')
            proc = subprocess.Popen(["/bin/bash", "-c", process1], bufsize=8192, stdin=None, stdout=logs, stderr=failedlogs, cwd=c)
            processes.append(proc)

        for p in processes:
            p.wait()
            # progress_bar.update()
        for c in cwd:
            proc = subprocess.Popen(["/bin/bash", "-c", process2], bufsize=8192, stdin=None,stdout=subprocess.PIPE,stderr=subprocess.PIPE, cwd=c)
        for c in cwd:
            exit = exists(os.path.expanduser(os.path.join(c+"/tfplan.json")))
        files = []
        for i in range(len(cwd)):
            if exists(os.path.expanduser(os.path.join(cwd[i], "tfplan.json"))):
                file = open(os.path.expanduser(os.path.join(cwd[i], "tfplan.json")))
                data = file.readlines()
                print(len(data))
                json_object = json.loads(data[0])
                file.close()
                files.append(json_object['resource_changes'])

        added = 0
        changed = 0
        deleted = 0

        for i in range(len(files)):
            print(bcolors.ENDC, "You re working in folder" , bcolors.OKBLUE, cwd[i])
            for j in files[i]:
                if j['change']['actions'] == ['create']:
                    added += 1
                    print(bcolors.OKGREEN, "+", bcolors.ENDC, ' ressource', j['address'], 'will be added')
                if j['change']['actions'] == ['update']:
                    changed += 1
                    print(bcolors.WARNING,"~", bcolors.ENDC, ' ressource', j['address'], 'will be updated in-place')
                if j['change']['actions'] == ['delete']:
                    deleted += 1
                    print(bcolors.FAIL ,"-", bcolors.ENDC, ' ressource', j['address'], 'will be destroyed')
                if changed == 0 and added == 0 and deleted == 0:
                    print(bcolors.OKGREEN +"  : No changes. Infrastructure is up-to-date in folder", bcolors.OKBLUE, cwd[i])
            print("Plan:", bcolors.OKGREEN, added, bcolors.ENDC,"to add, ", bcolors.WARNING,changed, bcolors.ENDC," to change, ",bcolors.FAIL,deleted,bcolors.ENDC," to destroy.")
        print("Plan:", bcolors.OKGREEN, added, bcolors.ENDC,"to add, ", bcolors.WARNING,changed, bcolors.ENDC," to change, ",bcolors.FAIL,deleted,bcolors.ENDC," to destroy.")




import json
import pandas as pd
import os
import subprocess
import csv
import terragit.terraConf as terraconf



class Compare:
    def __init__(self, gitlab_token, git_url):
        self.gitlab_token = gitlab_token
        self.git_url = git_url

    def list_state(self, infra, file1):

        cmd = "cd " + infra + " && terragrunt state list > ~/" + file1
        popen = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                 encoding='utf8')
        lines = popen.communicate(input='\n')[0].split("\n")

    def compare_state(self, infra1, infra2, file1, file2):
        self.list_state(infra1, file1)
        self.list_state(infra2, file2)
        fichier1 = open(os.path.expanduser(os.path.join("~/" + file1)), 'r')

        fichier1.close()

    def compare_tf_states(self, state1, state2):
        pwd =terraconf.TerraConf.current_path(self)
        pwd_f= pwd+"terragit-compare"

        folder_path=terraconf.TerraConf.goTo_or_create_folder(self,pwd_f)

        f = open(state1)

        data = f.read().rstrip()
        x = json.loads(data)

        df = pd.json_normalize(x, record_path=['resources'],
                               errors='ignore')

        f.close()
        f2 = open(state2)
        data2 = f2.read().rstrip()
        x = json.loads(data2)

        df2 = pd.json_normalize(x, record_path=['resources'],
                                errors='ignore')

        f2.close()

        l1 = []
        l23 = []
        State_list1 = ["source"]
        State_list = ["destination"]
        State_list3 = []
        new_list1 = []
        new_list = []
        for i in df.columns:
            l1.append([i, df[i].values])
        l2 = []
        for i in df2.columns:
            l2.append([i, df[i].values])

        l = [["object", "source", "destination"]]
        for i in l1:

            for j in l2:

                if (i[0] == j[0]):

                    l.append([i[0], i[1], j[1]])


        j=0
        while j<len(l[1][1]):
            cc=l[1][1][j]+"."+l[2][1][j]+"."+l[3][1][j]+"."+l[4][1][j]


            State_list.append(cc)

            j=j+1

        jr=0
        while jr<len(l[2][1]):

            cr=l[1][2][jr]+"."+l[2][2][jr]+"."+l[3][2][jr]+"."+l[4][2][jr]


            State_list1.append(cr)

            jr=jr+1



        for j in range(len(l[6][1])):

            l24=[["Property", "destination", "Source"]]
            jp=l[1][1][j]+"."+l[2][1][j]+"."+l[3][1][j]+"."+l[4][1][j]
            df22 = pd.json_normalize(l[6][1][j][0],
                                errors='ignore')

            df220 = pd.json_normalize(l[6][2][j][0],
                                     errors='ignore')


            for i in df22.columns:
                l24.append([i, df22[i].values, df220[i].values])
            self.write_tuple_liste_in_csv( l24, folder_path+'/S_list'+jp+'.csv')
        jj = 0
        new_list.append(State_list[0])
        for i in range(1, len(State_list)):

            l = State_list[0:jj]

            if State_list[i] not in l:

                new_list.append(State_list[i])
            else:

                continue
            jj += 1
        jk = 0
        new_list1.append(State_list1[0])
        for i in range(1, len(State_list1)):

            l = State_list1[0:jk]
            if i not in l:
                new_list1.append(State_list1[i])
            else:
                continue
            jk += 1

        for i in range(len(new_list)):
            State_list3.append([new_list[i], new_list1[i]])
        self.write_tuple_liste_in_csv( State_list3, folder_path+'/State_list.csv')


    def write_tuple_liste_in_csv(self, list, file):

        f_State_list = open(file, 'w')

        writer = csv.writer(f_State_list)

        writer.writerows(list)

        f_State_list.close()





import gitlab
import os
import sys
import subprocess
from random import randrange

class terradoc:
    def __init__(self,git_url,gitlab_token , project_id):
        self.git_url =git_url
        self.gitlab_token =gitlab_token
        self.project_id=project_id
        self.module=False
        self.live=False

    def docs(self , module , live):
        gl = gitlab.Gitlab(self.git_url,private_token = self.gitlab_token)
        project = gl.projects.get(self.project_id)
        gitUrlToClone =project.ssh_url_to_repo
        subprocess.call(['git', 'clone', gitUrlToClone] ,)

        projectpath=os.getcwd()
        os.chdir(projectpath+"/"+project.name.lower() )
        if module == True and live == False:
            random_suffix=randrange(0, 1000)
            os.environ['BRANCH_NAME'] = "docs"+str(random_suffix)
            commands = [ #instal Graphviz
               'curl -Lo ./terraform-docs.tar.gz https://github.com/terraform-docs/terraform-docs/releases/download/v0.15.0/terraform-docs-v0.15.0-$(uname)-amd64.tar.gz',
               'tar -xzf terraform-docs.tar.gz' ,
               'chmod +x terraform-docs' ,
               'git checkout -b $BRANCH_NAME' ,
               'terraform-docs markdown table --output-file README.md --output-mode inject .',
               'terraform init &&terraform graph |  sed -n \'/digraph/,$p\' | dot -Tsvg > graph.svg' ,
               'git add ./README.md' ,
               'git add ./graph.svg',
               'git commit --allow-empty -m "add docs"',
               'git push -u origin $BRANCH_NAME -o merge_request.create -o merge_request.target=master -o merge_request.title="add docs" -o merge_request.description="add docs" -o merge_request.remove_source_branch=true -o merge_request.merge_when_pipeline_succeeds=false']
        if module ==False and live ==True:
            random_suffix=randrange(0, 1000)
            os.environ['BRANCH_NAME'] = "graph"+str(random_suffix)
            commands = [ #instal Graphviz
               'git checkout -b $BRANCH_NAME' ,
               'terragrunt graph-dependencies |  sed -n \'/digraph/,$p\' | dot -Tsvg > graph.svg' ,
               'git add ./graph.svg',
               'git commit --allow-empty -m "add graph"',
               'git push -u origin $BRANCH_NAME -o merge_request.create -o merge_request.target=master -o merge_request.title="add graph" -o merge_request.description="add graph" -o merge_request.remove_source_branch=true -o merge_request.merge_when_pipeline_succeeds=false']

        count = 0
        for com in commands:
            os.system(com)
            count += 1
        else:
            print ("Finish..")











import requests  # token : gitlab_token
import os


class Clone:
    def __init__(self, gitlab_token, git_url, root_path):
        self.root_path = root_path
        self.gitlab_token = gitlab_token
        self.git_url = git_url

    def clone_projects(self, project_id, grp_id, path):
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        pwd=os.popen("pwd").read()
        if path != None:
            root_path = '~/'
        else:
            root_path = pwd.rstrip()+"/"

        if project_id != None:

            url_projets = self.git_url + "/api/v4/projects/" + str(project_id)
            project = requests.get(url_projets, headers=headers).json()
            path = root_path + "/" + project['path_with_namespace']

            if not os.path.isdir(path):
                os.makedirs(path)
                cmd = "git clone " + project['ssh_url_to_repo'] + " " + path
                os.system(cmd)

            else:
                print(f"{path} already exists")

        if grp_id != None:

            url_projets = self.git_url + "/api/v4/groups/" + str(grp_id) + "/projects"

            projets = requests.get(url_projets, headers=headers).json()
            for project in projets:

                path = root_path + project['path_with_namespace']
                if not os.path.isdir(path):
                    os.makedirs(path)
                    cmd = "git clone " + project['ssh_url_to_repo'] + " " + path
                    os.system(cmd)
                else:
                    print(f"{path} already exists")

            url_grps = self.git_url + "/api/v4/groups/" + str(grp_id) + "/subgroups"
            subgroups = requests.get(url_grps, headers=headers).json()
            for subgroup in subgroups:
                self.clone_projects(None, subgroup['id'])

    def list_project(self):
        info_list = []
        group_id = 0
        user_input = ''
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        url_group = self.git_url + "/api/v4/groups/2875/descendant_groups"
        groups = requests.get(url_group, headers=headers).json()

        name_list = []

        for g in groups:
            data = {}
            data['name'] = g['name']
            data['id'] = g['id']

            info_list.append(data)
            name_list.append(g['name'])

        input_message = "Pick an option:\n"
        for index, item in enumerate(name_list):
            input_message += f'{index+1}) {item}\n'
        input_message += 'Make a choice to clone group: '
        while user_input.lower() not in name_list:
            user_input = input(input_message)
        print('You picked: ' + user_input)

        for j in info_list:
            if user_input == j['name']:
                group_id = j['id']

        self.clone_projects(None, group_id, '~/')



import shutil
from terragit.terragrunt import *
import gitlab

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

logsFolder=""
pathList=[]
failedloglist=[]
printLogs=False

class apply_changes:
    def __init__(self):
        self.bcolor = bcolors

    def apply_ci(self):
        git_url="https://gitlab.allence.cloud"
        if len(sys.argv) > 1:
            git_url = sys.argv[1]
        logsFolder=pathlib.Path("logs").absolute().as_posix()
        idProject = os.environ.get('CI_PROJECT_ID')
        idCommit = os.environ.get('CI_COMMIT_SHA')
        gitlab_token = os.environ.get('gitlab_token')
        ci_commit_title = os.environ.get('CI_COMMIT_TITLE')

        gl = gitlab.Gitlab(url='https://gitlab.allence.cloud', private_token=gitlab_token)
        project = gl.projects.get(idProject)
        commit = project.commits.get(idCommit)
        diff = commit.diff()
        folderList = []

        if len(diff)==0:
            if not isdir(pathlib.Path(ci_commit_title).absolute().as_posix()):
                print(bcolors.FAIL+ci_commit_title +" is not valid path"+ bcolors.ENDC)
            else:
                ci_commit_titlePath=pathlib.Path(ci_commit_title).absolute().as_posix()
                printLogs=True
                self.apply(ci_commit_titlePath,logsFolder,printLogs)
        else:
            for change in diff:
                newPath=change['new_path']
                if not ("live/") in newPath:
                    print(pathlib.Path(newPath).absolute().as_posix()+bcolors.WARNING +" OUT of SCOPE"+ bcolors.ENDC)
                else:
                    pathh=pathlib.Path(newPath).parent.absolute().as_posix()
                    folderList.append(pathh)
        mylist = list(dict.fromkeys(folderList))
        TG_OUTPUT_LIMIT =3
        if os.environ.get("TG_OUTPUT_LIMIT")!=3 and os.environ.get("TG_OUTPUT_LIMIT")!=None :
            TG_OUTPUT_LIMIT = os.environ.get("TG_OUTPUT_LIMIT")
        if len(mylist)<=int(TG_OUTPUT_LIMIT):
            printLogs=True
        failedlogsFolder=pathlib.Path("failedlogs").absolute().as_posix()
        for path in mylist:
            if(isdir(path)):
                self.apply(path,logsFolder,printLogs)
        if failedloglist:
            for message in failedloglist:
                logfileName=message.split("live/")[1].replace("/","_")
                os.chdir(failedlogsFolder)
                shutil.move(logsFolder+"/"+logfileName+".log", "failed_"+logfileName+".log")
            sys.exit(1)

    def apply(self,path,logsFolder,printLogs):
        if  path in pathList:
            return
        else:
            pathList.append(path)
            only_tf_files=[f for f in  sorted(listdir(path)) if not isdir(join(path, f)) and f.endswith('.tf')]
            if len(only_tf_files)==0:
                onlyDirectories = [d for d in sorted(listdir(path)) if isdir(join(path, d)) and d !=".terraform" and d != ".terragrunt-cache"]
                if(len(onlyDirectories) > 0):
                    for i in range(0, len(onlyDirectories)):
                        self.apply(path+"/"+onlyDirectories[i],logsFolder,printLogs)
                return
            os.chdir(path)
            logfileName=path.split("live/")[1].replace("/","_")
            state_list = ("terragrunt apply -no-color 2>&1 | tee " +logsFolder+"/"+logfileName+".log")
            popen = subprocess.Popen(state_list, stdout = subprocess.PIPE, shell = True, encoding = 'utf8')
            lines = popen.stdout.readlines()
            popen.stdout.close()
            print(bcolors.OKBLUE+path+":" +bcolors.ENDC)
            for line in lines:
                if (printLogs):
                    print(line)
                if ("Hit multiple errors" in line):
                    lint=lines[(lines.index(line)):(len(lines))]
                    for l in lint:
                        print(l.replace('\n',''))
                    print(bcolors.FAIL +" COULDNT PROCESS"+ bcolors.ENDC)
                    failedloglist.append(path)
                    break
                if("Apply complete! Resources:" in line):
                    print(bcolors.OKGREEN + line + bcolors.ENDC)
                    continue











from setuptools import setup, find_packages
 
classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]
 
setup(
  name='terragit',
  version='0.3.42',
  description='terragit package',
  long_description=open('README.md').read(),
  long_description_content_type="text/markdown",
  url='https://gitlab.com/commons-acp/python/terraform-gitlab',
  author='aminnn',
  author_email='trabelsiamin9@gmail.com',
  license='MIT', 
  classifiers=classifiers,
  keywords='terragit',
  packages=find_packages(),
  install_requires=['pandas'],
    entry_points = ({
          'console_scripts': [
              'terragit = terragit.__main__:main'
          ]
      })
)



import requests


class KeybaseFunctions:
    def __init__(self, ):
        self.headers = {'Content-Type': 'application/json'}

    def check_keybase_username_existance(self, username):
        url = "https://keybase.io/_/api/1.0/user/lookup.json?usernames=" + username
        return requests.get(url, headers=self.headers).json()







import shutil
from terragit.terragrunt import *
import gitlab
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

pathList=[]
failedloglist=[]
logsFolder=""
printLogs=False

class mr_changes:
    def __init__(self):
        self.bcolor = bcolors

    def plan_ci(self):

        git_url='https://gitlab.allence.cloud'
        if len(sys.argv) > 1:
            git_url = sys.argv[1]
        logsFolder = pathlib.Path("logs").absolute().as_posix()
        failedlogsFolder = pathlib.Path("failedlogs").absolute().as_posix()
        idMr = os.environ.get('CI_MERGE_REQUEST_IID')
        idProject = os.environ.get('CI_PROJECT_ID')
        gitlab_token = os.environ.get('gitlab_token')

        ci_commit_title = os.environ.get('CI_COMMIT_TITLE')

        gl = gitlab.Gitlab(url='https://gitlab.allence.cloud',private_token=gitlab_token)

        project = gl.projects.get(idProject)

        mr = project.mergerequests.get(idMr)
        folderList = []
        mrchange = mr.changes()
        changes = mrchange['changes']

        if len(changes) == 0:
            if not isdir(pathlib.Path(ci_commit_title).absolute().as_posix()):
                print(bcolors.FAIL+ci_commit_title+" is not valid path" + bcolors.ENDC)
                failedloglist.append(ci_commit_title)
            else:
                ci_mr_titlePath=pathlib.Path(ci_commit_title).absolute().as_posix()
                printLogs=True
                self.tg_plan(ci_mr_titlePath,logsFolder,printLogs)
        else:
            for change in changes:
                newPath=change['new_path']
                if not ("live/") in newPath:
                    print(pathlib.Path(newPath).absolute().as_posix()+bcolors.WARNING +" OUT of SCOPE"+ bcolors.ENDC)
                else:
                    pathh=pathlib.Path(newPath).parent.absolute().as_posix()
                    folderList.append(pathh)
        mylist = list(dict.fromkeys(folderList))
        TG_OUTPUT_LIMIT =3
        if os.environ.get("TG_OUTPUT_LIMIT")!=3 and os.environ.get("TG_OUTPUT_LIMIT")!=None :
            TG_OUTPUT_LIMIT = os.environ.get("TG_OUTPUT_LIMIT")
        if len(mylist)<=int(TG_OUTPUT_LIMIT):
            printLogs=True
        for path in mylist:
            if isdir(path):
                self.tg_plan(path,logsFolder,printLogs)
        if failedloglist:
            for message in failedloglist:
                logfileName=message.split("live/")[1].replace("/","_")
                os.chdir(failedlogsFolder)
                shutil.move(logsFolder+"/"+logfileName+".log", "failed_"+logfileName+".log")
            sys.exit(1)

    def tg_plan(self,path,logsFolder,printLogs):
        if path in pathList:
            return
        else:
            pathList.append(path)
            only_tf_files=[f for f in  sorted(listdir(path)) if not isdir(join(path, f)) and f.endswith('.tf')]
            if len(only_tf_files)==0:
                onlyDirectories = [d for d in sorted(listdir(path)) if isdir(join(path, d)) and d !=".terraform" and d != ".terragrunt-cache"]
                if(len(onlyDirectories) > 0):
                    for i in range(0, len(onlyDirectories)):
                        self.tg_plan(path+"/"+onlyDirectories[i],logsFolder,printLogs)
                return
            os.chdir(path)
            logfileName=path.split("live/")[1].replace("/","_")

            state_list = ("terragrunt plan -no-color 2>&1 | tee " +logsFolder+"/"+logfileName+".log")
            popen = subprocess.Popen(state_list, stdout = subprocess.PIPE, shell = True, encoding = 'utf8')
            lines = popen.stdout.readlines()
            popen.stdout.close()
            print(bcolors.OKBLUE+path+":" +bcolors.ENDC)
            for line in lines:
                if (printLogs):
                    print(line)
                else:
                    if ("will be updated in-place" in line):
                        print(line.replace('\n',''))
                    if ("will be created" in line):
                        print(line.replace('\n',''))
                    if ("must be replaced" in line):
                        print(line.replace('\n',''))

                        lint=lines[(lines.index(line)):(len(lines))]
                        lin=lint[0:(lint.index('\n'))]
                        for l in lin:
                            if ("forces replacement" in l):
                                print(l.replace('\n',''))
                                print(lin[(lin.index(l))+1])
                    if ("will be destroyed" in line):
                        print(line.replace('\n',''))
                    if (line.startswith("Error")):
                        lint=lines[(lines.index(line)):(len(lines))]
                        for l in lint:
                            print(l.replace('\n',''))
                        print(bcolors.FAIL +" COULDNT PROCESS"+ bcolors.ENDC)
                        failedloglist.append(path)
                        return
                if("No changes. Infrastructure is up-to-date." in line):
                    print(bcolors.OKGREEN +"  : No changes. Infrastructure is up-to-date."+ bcolors.ENDC)
                    return
                if("Plan:" in line):
                    print(bcolors.WARNING +line+ bcolors.ENDC)
                    return
            print(bcolors.FAIL +" COULDNT PROCESS"+ bcolors.ENDC)
            failedloglist.append(path)







